/*
 This file is part of the Greenfoot program. 
 Copyright (C) 2005-2009,2010  Poul Henriksen and Michael Kolling 
 
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 
 of the License, or (at your option) any later version. 
 
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 
 This file is subject to the Classpath exception as provided in the  
 LICENSE.txt file that accompanied this code.
 */
package org.droidfoot.gui;

/**
 * Interface for classes that wants to be able to receive events from the
 * InputManager.
 * 
 * <p>
 * You should not use this interface directly, but rather one of the sub
 * interfaces.
 * 
 * @see TriggeredKeyListener
 * @see TriggeredMouseListener
 * @see TriggeredMouseMotionListener
 * 
 * @author Poul Henriksen
 */
public interface TriggeredListener {
	/**
	 * Fired when this listener will start receiving events.
	 * 
	 * @param obj
	 *            Optional object to be passed to the listener. If a state
	 *            change occurred due to an object being constructed, this will
	 *            be a reference to the constructed object. Can otherwise be
	 *            null.
	 */
	public void listeningStarted(Object obj);

	/**
	 * Fired when this event will stop receiving events.
	 * 
	 */
	public void listeningEnded();
}
